@@ -23,6 +23,7 @@ XTF_DURING_LOAD = 0x01
 XTF_TRY_LOAD = 0x02
 XTF_LOAD_MUST_SUCCEED = 0x03
 
+
 XTOPT_INVERT = 1 << 0
 XTOPT_MAND = 1 << 1
 XTOPT_MULTI = 1 << 2
@@ -386,7 +387,8 @@ class _xtables_match_v10(ct.Structure):
                 ("save", ct.CFUNCTYPE(None, ct.c_void_p,
                                       ct.POINTER(xt_entry_match))),
                 # Print match name or alias
-                ("alias", ct.CFUNCTYPE(ct.c_char_p, ct.POINTER(xt_entry_match))),
+                ("alias", ct.CFUNCTYPE(ct.c_char_p,
+                                       ct.POINTER(xt_entry_match))),
                 # pointer to list of extra command-line options
                 ("extra_opts", ct.POINTER(option)),
 
@@ -635,7 +637,8 @@ class _xtables_target_v10(ct.Structure):
                 ("save", ct.CFUNCTYPE(None, ct.c_void_p,
                                       ct.POINTER(xt_entry_target))),
                 # Print target name or alias
-                ("alias", ct.CFUNCTYPE(ct.c_char_p, ct.POINTER(xt_entry_target))),
+                ("alias", ct.CFUNCTYPE(ct.c_char_p,
+                                       ct.POINTER(xt_entry_target))),
                 # pointer to list of extra command-line options
                 ("extra_opts", ct.POINTER(option)),
 
@@ -677,7 +680,7 @@ _libc, _ = find_library("c")
 _optind = ct.c_long.in_dll(_libc, "optind")
 _optarg = ct.c_char_p.in_dll(_libc, "optarg")
 
-_lib_xtables, _xtables_version = find_library("xtables")
+_lib_xtables, xtables_version = find_library("xtables")
 _xtables_libdir = os.getenv("XTABLES_LIBDIR")
 if _xtables_libdir is None:
     import os.path
@@ -762,14 +765,14 @@ class xtables(object):
     _xtables_xt_params = ct.c_void_p.in_dll(_lib_xtables, "xt_params")
     _xtables_matches = (ct.c_void_p.in_dll(_lib_xtables, "xtables_matches"))
     try:
-        _xtables_pending_matches = (ct.c_void_p.in_dll(_lib_xtables,
-                                                       "xtables_pending_matches"))
+        _xtables_pending_matches = (ct.c_void_p.in_dll(
+            _lib_xtables, "xtables_pending_matches"))
     except ValueError:
         _xtables_pending_matches = ct.POINTER(None)
     _xtables_targets = (ct.c_void_p.in_dll(_lib_xtables, "xtables_targets"))
     try:
-        _xtables_pending_targets = (ct.c_void_p.in_dll(_lib_xtables,
-                                                       "xtables_pending_targets"))
+        _xtables_pending_targets = (ct.c_void_p.in_dll(
+            _lib_xtables, "xtables_pending_targets"))
     except ValueError:
         _xtables_pending_targets = ct.POINTER(None)
 
@@ -800,16 +803,16 @@ class xtables(object):
         self._xt_globals.exit_err = _xt_exit
 
         thismodule = sys.modules[__name__]
-        matchname = "_xtables_match_v%d" % (_xtables_version)
-        targetname = "_xtables_target_v%d" % (_xtables_version)
+        matchname = "_xtables_match_v%d" % (xtables_version)
+        targetname = "_xtables_target_v%d" % (xtables_version)
         try:
             self._match_struct = getattr(thismodule, matchname)
             self._target_struct = getattr(thismodule, targetname)
         except:
             raise XTablesError("unknown xtables version %d" %
-                               (_xtables_version))
+                               (xtables_version))
 
-        self._loaded_exts = []
+        self._loaded_exts = set()
 
         # make sure we're initializing with clean state
         self._xt_params = ct.c_void_p(None).value
@@ -857,7 +860,7 @@ class xtables(object):
         return name
 
     def _loaded(self, name):
-        self._loaded_exts.append(name)
+        self._loaded_exts.add(name)
 
     def _is_loaded(self, name):
         if name in self._loaded_exts:
@@ -898,6 +901,8 @@ class xtables(object):
             raise XTablesError("Unknown protocol %d" % (self.proto))
 
     def _try_register(self, name):
+        if self._is_loaded(name):
+            return
         if isinstance(name, bytes):
             name = name.decode()
         if self._try_extinit(name, _lib_xtables):
@@ -917,12 +922,13 @@ class xtables(object):
         match = xtables._xtables_find_match(name, XTF_TRY_LOAD, None)
         if not match:
             self._try_register(name)
-            match = xtables._xtables_find_match(name, XTF_TRY_LOAD, None)
+            match = xtables._xtables_find_match(name, XTF_DONT_LOAD, None)
             if not match:
                 return match
-        self._loaded(name)
 
-        return ct.cast(match, ct.POINTER(self._match_struct))
+        m = ct.cast(match, ct.POINTER(self._match_struct))
+        self._loaded(m[0].name)
+        return m
 
     @preserve_globals
     def find_target(self, name):
@@ -932,12 +938,13 @@ class xtables(object):
         target = xtables._xtables_find_target(name, XTF_TRY_LOAD)
         if not target:
             self._try_register(name)
-            target = xtables._xtables_find_target(name, XTF_TRY_LOAD)
+            target = xtables._xtables_find_target(name, XTF_DONT_LOAD)
             if not target:
                 return target
-        self._loaded(name)
 
-        return ct.cast(target, ct.POINTER(self._target_struct))
+        t = ct.cast(target, ct.POINTER(self._target_struct))
+        self._loaded(t[0].name)
+        return t
 
     @preserve_globals
     def save(self, module, ip, ptr):
